<?php
/**
 * DBCart 商城系统
 *
 * ==========================================================================
 * @link      https://www.dbcart.cn/
 * @copyright 北京珑大钜商科技有限公司，并保留所有权利。
 * @license   https://www.dbcart.cn/license.html License
 * ==========================================================================
 *
 * @author    静静的风 <baron@loongdom.cn>
 *
 */

namespace Admin\Controller;

use Admin\Entity\AdminUserGroup;
use Admin\Entity\AdminUserGroupContent;
use Admin\Form\AdminUserGroupForm;
use Admin\Service\AdminUserGroupContentManager;
use Admin\Service\AdminUserGroupManager;
use Admin\Service\AdminUserManager;
use Doctrine\ORM\EntityManager;
use Laminas\Mvc\I18n\Translator;

class AdminGroupController extends AdminActionController
{
    private $translator;
    private $entityManager;
    private $adminUserManager;
    private $adminGroupManager;
    private $permissionArray;
    private $adminUserGroupContentManager;

    public function __construct(
        Translator              $translator,
        EntityManager           $entityManager,
        AdminUserManager        $adminUserManager,
        AdminUserGroupManager   $adminGroupManager,
        AdminUserGroupContentManager $adminUserGroupContentManager,
        $permissionArray
    )
    {
        $this->translator       = $translator;
        $this->entityManager    = $entityManager;
        $this->adminUserManager = $adminUserManager;
        $this->adminGroupManager= $adminGroupManager;
        $this->adminUserGroupContentManager = $adminUserGroupContentManager;

        $this->permissionArray = $permissionArray;
    }

    /**
     * 直接跳转
     * @return \Laminas\Http\Response|\Laminas\View\Model\ViewModel
     */
    public function indexAction()
    {
        return $this->redirect()->toRoute('admin-group', ['action'=> 'adminGroupList']);
    }

    /**
     * 管理员组列表
     * @return array
     */
    public function adminGroupListAction(): array
    {
        $adminUserGroup = $this->entityManager->getRepository(AdminUserGroup::class)->findAdminUserGroupList($this->adminCommon()->adminLanguageCode());

        return ['adminGroup'=>$adminUserGroup];
    }

    /**
     * 添加管理员组
     * @return array|\Laminas\Http\Response
     * @throws \Doctrine\ORM\ORMException
     * @throws \Doctrine\ORM\OptimisticLockException
     */
    public function addAdminGroupAction()
    {
        $array = [];

        $array['form'] = new AdminUserGroupForm();
        $form = $array['form'];

        $array['permissionArray'] = $this->permissionArray;
        $valueArray = [];
        foreach ($this->permissionArray as $key => $permission) {
            foreach ($permission['controllers'] as $controllerKey => $controllerValue) {
                foreach ($controllerValue['action'] as $value) {
                    $valueArray[str_replace('\\', '_', $controllerKey). '_' . $value] = $controllerValue['actionNames'][$value];
                }
            }
        }
        $form->get('adminGroupPermission')->setValueOptions($valueArray);

        if($this->getRequest()->isPost()) {
            $data = $this->params()->fromPost();
            $form->setData($data);
            if($form->isValid()) {
                $data = $form->getData();

                $adminGroup = $this->adminGroupManager->addAdminGroup($data);
                $this->adminUserGroupContentManager->addOrEditAdminUserGroupContent($data, $adminGroup->getAdminGroupId());

                $this->adminCommon()->addOperLog(sprintf($this->translator->translate('管理员组 %s 添加成功!'), $data['adminGroupName'.$this->adminCommon()->adminLanguageCode()]), $this->translator->translate('管理员组'));

                return $this->redirect()->toRoute('admin-group', ['action'=> 'adminGroupList']);
            }
        }

        return $array;
    }

    /**
     * 编辑管理员组
     * @return array|\Laminas\Http\Response
     * @throws \Doctrine\ORM\ORMException
     * @throws \Doctrine\ORM\OptimisticLockException
     */
    public function editAdminGroupAction()
    {
        $adminGroupId = (int) $this->params()->fromRoute('id', -1);

        $adminGroup = $this->entityManager->getRepository(AdminUserGroup::class)->findOneByAdminGroupId($adminGroupId);
        if($adminGroup == null) {
            $this->flashMessenger()->addWarningMessage($this->translator->translate('该管理员组不存在!'));
            return $this->redirect()->toRoute('admin-group', ['action'=> 'adminGroupList']);
        }

        $form = new AdminUserGroupForm();

        $valueArray = [];
        foreach ($this->permissionArray as $key => $permission) {
            foreach ($permission['controllers'] as $controllerKey => $controllerValue) {
                foreach ($controllerValue['action'] as $value) {
                    $valueArray[str_replace('\\', '_', $controllerKey). '_' . $value] = $controllerValue['actionNames'][$value];
                }
            }
        }
        $form->get('adminGroupPermission')->setValueOptions($valueArray);

        if($this->getRequest()->isPost()) {
            $data = $this->params()->fromPost();
            $form->setData($data);
            if($form->isValid()) {
                $data = $form->getData();

                $this->adminGroupManager->updateAdminGroup($adminGroup, $data);
                $this->adminUserGroupContentManager->addOrEditAdminUserGroupContent($data, $adminGroupId);

                $this->adminCommon()->addOperLog(sprintf($this->translator->translate('管理员组 %s 编辑成功!'), $data['adminGroupName'.$this->adminCommon()->adminLanguageCode()]), $this->translator->translate('管理员组'));

                return $this->redirect()->toRoute('admin-group', ['action'=> 'adminGroupList']);
            }
        } else {
            $form->setData(['adminGroupId'  => $adminGroup->getAdminGroupId()]);

            $adminContent = $this->entityManager->getRepository(AdminUserGroupContent::class)->findBy(['adminGroupId' => $adminGroupId]);
            foreach ($adminContent as $contentValue) {
                if (!in_array($contentValue->getLanguageCode(), ADMIN_LANGUAGE)) continue;
                $form->setData(['adminGroupName'.$contentValue->getLanguageCode() => $contentValue->getAdminGroupName()]);
            }
        }

        return ['group'=>$adminGroup, 'form'=>$form, 'permissionArray' => $this->permissionArray];
    }

    /**
     * 删除管理员组
     * @return \Laminas\Http\Response
     * @throws \Doctrine\ORM\ORMException
     * @throws \Doctrine\ORM\OptimisticLockException
     */
    public function deleteAdminGroupAction()
    {
        if(!$this->adminCommon()->validatorCsrf()) return $this->redirect()->toRoute('admin-group', ['action'=> 'adminGroupList']);

        $adminGroupId = (int) $this->params()->fromRoute('id', -1);
        if($adminGroupId == 1) {
            $this->flashMessenger()->addErrorMessage($this->translator->translate('该管理员组不能删除!'));
            return $this->redirect()->toRoute('admin-group', ['action'=> 'adminGroupList']);
        }

        $adminGroup = $this->entityManager->getRepository(AdminUserGroup::class)->findOneByAdminGroupId($adminGroupId);
        if($adminGroup == null) {
            $this->flashMessenger()->addWarningMessage($this->translator->translate('该管理员组不存在!'));
            return $this->redirect()->toRoute('admin-group', ['action'=> 'adminGroupList']);
        }

        $adminGroupContent = $this->entityManager->getRepository(AdminUserGroupContent::class)->findOneBy(['adminGroupId' => $adminGroupId, 'languageCode' => $this->adminCommon()->adminLanguageCode()]);
        $delState = $this->adminGroupManager->deleteAdminGroup($adminGroup);
        $this->adminUserGroupContentManager->deleteAdminUserGroupContent($adminGroupId);
        if($delState) {
            $this->adminCommon()->addOperLog(sprintf($this->translator->translate('管理员组 %s 删除成功!'), $adminGroupContent->getAdminGroupName()), $this->translator->translate('管理员组'));
        }
        else $this->flashMessenger()->addErrorMessage(sprintf($this->translator->translate('管理员组 %s 删除失败!该组不存在或该组下有管理员'), $adminGroupContent->getAdminGroupName()));

        return $this->adminCommon()->toReferer();
    }
}